"""Test for the Tasmania Fire Service Incidents feed."""
import datetime
import unittest
from unittest import mock

from georss_client import UPDATE_OK
from georss_client.tasmania_fire_service_incidents_feed import TfsIncidentsFeed
from tests.utils import load_fixture


class TestTfsIncidentsFeed(unittest.TestCase):
    """Test the Tasmania Fire Service Incidents feed."""

    @mock.patch("requests.Request")
    @mock.patch("requests.Session")
    def test_update_ok(self, mock_session, mock_request):
        """Test updating feed is ok."""
        home_coordinates = (-42.0, 147.0)
        mock_session.return_value.__enter__.return_value.send\
            .return_value.ok = True
        mock_session.return_value.__enter__.return_value.send\
            .return_value.text = \
            load_fixture('tasmania_fire_service_incidents_feed.xml')

        feed = TfsIncidentsFeed(home_coordinates)
        assert repr(feed) == "<TfsIncidentsFeed(home=(-42.0, 147.0), " \
                             "url=http://www.fire.tas.gov.au/Show?pageId=" \
                             "colBushfireSummariesRss, radius=None, " \
                             "categories=None)>"
        status, entries = feed.update()
        assert status == UPDATE_OK
        self.assertIsNotNone(entries)
        assert len(entries) == 2

        feed_entry = entries[0]
        assert feed_entry.title == "Title 1"
        assert feed_entry.external_id == "1234"
        assert feed_entry.coordinates == (-42.123456, 147.234567)
        self.assertAlmostEqual(feed_entry.distance_to_home, 23.7, 1)
        assert feed_entry.published \
            == datetime.datetime(2018, 9, 29, 15, 30,
                                 tzinfo=datetime.timezone.utc)
        assert feed_entry.location == "Location 1"
        assert feed_entry.region == "Region 1"
        assert feed_entry.responsible_agency == "Agency 1"
        assert feed_entry.status == "Status 1"
        assert feed_entry.size == "Size 1"
        assert feed_entry.type == "Type 1"
        assert feed_entry.attribution == "Tasmania Fire Service"
        assert repr(feed_entry) == "<TfsIncidentsFeedEntry(id=1234)>"

        feed_entry = entries[1]
        assert feed_entry.title == "Title 2"
        self.assertIsNone(feed_entry.published)

    @mock.patch("requests.Request")
    @mock.patch("requests.Session")
    def test_update_ok_with_category(self, mock_session, mock_request):
        """Test updating feed is ok."""
        home_coordinates = (-42.0, 147.0)
        mock_session.return_value.__enter__.return_value.send\
            .return_value.ok = True
        mock_session.return_value.__enter__.return_value.send\
            .return_value.text = \
            load_fixture('tasmania_fire_service_incidents_feed.xml')

        feed = TfsIncidentsFeed(home_coordinates,
                                filter_categories=['Category 1'])
        status, entries = feed.update()
        assert status == UPDATE_OK
        self.assertIsNotNone(entries)
        assert len(entries) == 1

        feed_entry = entries[0]
        assert feed_entry.title == "Title 1"
        assert feed_entry.external_id == "1234"



"""Test for the Department of Fire and Emergency Services (DFES) feed."""
import datetime
import unittest
from unittest import mock

from georss_client import UPDATE_OK
from georss_client.exceptions import GeoRssException
from georss_client.wa_dfes_feed import WaDfesFeed
from tests.utils import load_fixture


class TestWaDfesFeed(unittest.TestCase):
    """Test the Department of Fire and Emergency Services (DFES) feed."""

    @mock.patch("requests.Request")
    @mock.patch("requests.Session")
    def test_update_ok_warnings(self, mock_session, mock_request):
        """Test updating feed is ok."""
        home_coordinates = (-31.0, 121.0)
        mock_session.return_value.__enter__.return_value.send\
            .return_value.ok = True
        mock_session.return_value.__enter__.return_value.send\
            .return_value.text = \
            load_fixture('wa_dfes_warnings_feed.xml')

        feed = WaDfesFeed(home_coordinates, 'warnings')
        assert repr(feed) == "<WaDfesFeed(home=(-31.0, 121.0), " \
                             "url=https://www.emergency.wa.gov.au/data/" \
                             "message.rss, radius=None, " \
                             "categories=None)>"
        status, entries = feed.update()
        assert status == UPDATE_OK
        self.assertIsNotNone(entries)
        assert len(entries) == 2

        feed_entry = entries[0]
        assert feed_entry.title == "Title 1"
        assert feed_entry.external_id == "1234"
        assert feed_entry.coordinates == (-30.97304, 121.30196)
        self.assertAlmostEqual(feed_entry.distance_to_home, 28.9, 1)
        assert feed_entry.published \
            == datetime.datetime(2018, 9, 30, 8, 30,
                                 tzinfo=datetime.timezone.utc)
        assert feed_entry.category == "Category 1"
        assert feed_entry.region == "Region 1"
        assert feed_entry.attribution == "Department of Fire and Emergency " \
                                         "Services"
        assert repr(feed_entry) == "<WaDfesWarningsFeedEntry(id=1234)>"

        feed_entry = entries[1]
        assert feed_entry.title == "Title 2"
        self.assertIsNone(feed_entry.published)

    @mock.patch("requests.Request")
    @mock.patch("requests.Session")
    def test_update_ok_warnings_with_category(self, mock_session,
                                              mock_request):
        """Test updating feed is ok."""
        home_coordinates = (-31.0, 121.0)
        mock_session.return_value.__enter__.return_value.send\
            .return_value.ok = True
        mock_session.return_value.__enter__.return_value.send\
            .return_value.text = \
            load_fixture('wa_dfes_warnings_feed.xml')

        feed = WaDfesFeed(home_coordinates, 'warnings',
                          filter_categories=['Category 1'])
        status, entries = feed.update()
        assert status == UPDATE_OK
        self.assertIsNotNone(entries)
        assert len(entries) == 1

        feed_entry = entries[0]
        assert feed_entry.title == "Title 1"
        assert feed_entry.external_id == "1234"

    @mock.patch("requests.Request")
    @mock.patch("requests.Session")
    def test_update_ok_all_incidents(self, mock_session, mock_request):
        """Test updating feed is ok."""
        home_coordinates = (-31.0, 121.0)
        mock_session.return_value.__enter__.return_value.send\
            .return_value.ok = True
        mock_session.return_value.__enter__.return_value.send\
            .return_value.text = \
            load_fixture('wa_dfes_all_incidents_feed.xml')

        feed = WaDfesFeed(home_coordinates, 'all_incidents')
        assert repr(feed) == "<WaDfesFeed(home=(-31.0, 121.0), " \
                             "url=https://www.emergency.wa.gov.au/data/" \
                             "incident_FCAD.rss, radius=None, " \
                             "categories=None)>"
        status, entries = feed.update()
        assert status == UPDATE_OK
        self.assertIsNotNone(entries)
        assert len(entries) == 2

        feed_entry = entries[0]
        assert feed_entry.title == "Title 1"
        assert feed_entry.external_id == "1234"
        assert feed_entry.coordinates == (-23.12641, 119.94800)
        self.assertAlmostEqual(feed_entry.distance_to_home, 881.7, 1)
        assert feed_entry.published \
            == datetime.datetime(2018, 9, 30, 8, 30)
        assert feed_entry.category == "Category 1"
        assert feed_entry.region == "Region 1"
        assert feed_entry.attribution == "Department of Fire and Emergency " \
                                         "Services"
        assert repr(feed_entry) == "<WaDfesAllIncidentsFeedEntry(id=1234)>"

        feed_entry = entries[1]
        assert feed_entry.title == "Title 2"
        self.assertIsNone(feed_entry.published)

    @mock.patch("requests.Request")
    @mock.patch("requests.Session")
    def test_update_ok_all_incidents_with_category(self, mock_session,
                                                   mock_request):
        """Test updating feed is ok."""
        home_coordinates = (-31.0, 121.0)
        mock_session.return_value.__enter__.return_value.send\
            .return_value.ok = True
        mock_session.return_value.__enter__.return_value.send\
            .return_value.text = \
            load_fixture('wa_dfes_all_incidents_feed.xml')

        feed = WaDfesFeed(home_coordinates, 'all_incidents',
                          filter_categories=['Category 1'])
        status, entries = feed.update()
        assert status == UPDATE_OK
        self.assertIsNotNone(entries)
        assert len(entries) == 1

        feed_entry = entries[0]
        assert feed_entry.title == "Title 1"
        assert feed_entry.external_id == "1234"

    def test_update_wrong_feed(self):
        """Test invalid feed name."""
        home_coordinates = (-31.0, 121.0)

        with self.assertRaises(GeoRssException):
            WaDfesFeed(home_coordinates, 'DOES NOT EXIST')



"""
WA Department of Fire and Emergency Services (DFES) Feed.

Fetches GeoRSS feed from WA Department of Fire and Emergency Services (DFES)
Feed.
"""
import logging
from datetime import datetime

from typing import Optional

from georss_client import GeoRssFeed, FeedEntry
from georss_client.consts import CUSTOM_ATTRIBUTE
from georss_client.exceptions import GeoRssException

_LOGGER = logging.getLogger(__name__)

ADDITIONAL_NAMESPACES = {
    'http://emergency.wa.gov.au/xmlns/dfes': 'dfes'
}

ATTRIBUTION = "Department of Fire and Emergency Services"

REGEXP_ATTR_CATEGORY_WARNINGS = '<b>Category: </b>(?P<{}>[^<]+)</div>'\
    .format(CUSTOM_ATTRIBUTE)
REGEXP_ATTR_CATEGORY_ALL_INCIDENTS = '^(?P<{}>[^<]+) <'\
    .format(CUSTOM_ATTRIBUTE)
REGEXP_ATTR_REGION = '<region>(?P<{}>[^<]+)</region>'\
    .format(CUSTOM_ATTRIBUTE)

URL_PREFIX = 'https://www.emergency.wa.gov.au/data/'
URLS = {
    'warnings': URL_PREFIX + 'message.rss',
    'all_incidents': URL_PREFIX + 'incident_FCAD.rss',
}

XML_TAG_DFES_REGION = 'dfes:region'


class WaDfesFeed(GeoRssFeed):
    """Department of Fire and Emergency Services (DFES) feed."""

    def __init__(self, home_coordinates, feed, filter_radius=None,
                 filter_categories=None):
        """Initialise this service."""
        if feed in URLS:
            super().__init__(home_coordinates, URLS[feed],
                             filter_radius=filter_radius,
                             filter_categories=filter_categories)
            self._feed = feed
        else:
            _LOGGER.error("Unknown feed category %s", feed)
            raise GeoRssException("Feed category must be one of %s".format(
                URLS.keys()))

    def _new_entry(self, home_coordinates, rss_entry, global_data):
        """Generate a new entry."""
        if self._feed == 'warnings':
            return WaDfesWarningsFeedEntry(home_coordinates, rss_entry)
        if self._feed == 'all_incidents':
            return WaDfesAllIncidentsFeedEntry(home_coordinates, rss_entry)

    def _additional_namespaces(self):
        """Provide additional namespaces, relevant for this feed."""
        return ADDITIONAL_NAMESPACES

    def _filter_entries(self, entries):
        """Filter the provided entries."""
        entries = super()._filter_entries(entries)
        if self._filter_categories:
            return list(filter(lambda entry:
                               entry.category in self._filter_categories,
                               entries))
        return entries


class WaDfesFeedEntry(FeedEntry):
    """Department of Fire and Emergency Services (DFES) feed entry."""

    def __init__(self, home_coordinates, rss_entry):
        """Initialise this service."""
        super().__init__(home_coordinates, rss_entry)

    @property
    def attribution(self) -> str:
        """Return the attribution of this entry."""
        return ATTRIBUTION

    @property
    def published(self) -> Optional[datetime]:
        """Return the published date of this entry."""
        if self._rss_entry:
            return self._rss_entry.published_date
        return None


class WaDfesWarningsFeedEntry(WaDfesFeedEntry):
    """Department of Fire and Emergency Services (DFES) Warnings feed entry."""

    @property
    def category(self) -> str:
        """Return the type of this entry."""
        return self._search_in_description(REGEXP_ATTR_CATEGORY_WARNINGS)

    @property
    def region(self) -> Optional[str]:
        """Return the region of this entry."""
        if self._rss_entry:
            return self._rss_entry.get_additional_attribute(
                XML_TAG_DFES_REGION)
        return None


class WaDfesAllIncidentsFeedEntry(WaDfesFeedEntry):
    """Department of Fire and Emergency Services (DFES) All Incidents feed
    entry."""

    @property
    def category(self) -> str:
        """Return the type of this entry."""
        return self._search_in_description(REGEXP_ATTR_CATEGORY_ALL_INCIDENTS)

    @property
    def region(self) -> str:
        """Return the region of this entry."""
        return self._search_in_description(REGEXP_ATTR_REGION)



"""
Tasmania Fire Service Incidents Feed.

Fetches GeoRSS feed from Tasmania Fire Service Incidents Feed.
"""
from datetime import datetime

from typing import Optional

from georss_client import GeoRssFeed, FeedEntry
from georss_client.consts import CUSTOM_ATTRIBUTE

ATTRIBUTION = "Tasmania Fire Service"

REGEXP_ATTR_LOCATION = 'LOCATION: (?P<{}>[^<]+)<br />'\
    .format(CUSTOM_ATTRIBUTE)
REGEXP_ATTR_REGION = 'Region: (?P<{}>[^<]+)<br />'\
    .format(CUSTOM_ATTRIBUTE)
REGEXP_ATTR_RESPONSIBLE_AGENCY = 'RESPONSIBLE AGENCY: (?P<{}>[^<]+)<br />'\
    .format(CUSTOM_ATTRIBUTE)
REGEXP_ATTR_SIZE = 'SIZE: (?P<{}>[^<]+)<br />'\
    .format(CUSTOM_ATTRIBUTE)
REGEXP_ATTR_STATUS = 'STATUS: (?P<{}>[^<]+)<br />'\
    .format(CUSTOM_ATTRIBUTE)
REGEXP_ATTR_TYPE = 'TYPE: (?P<{}>[^<]+)<br />'\
    .format(CUSTOM_ATTRIBUTE)

URL = "http://www.fire.tas.gov.au/Show?pageId=colBushfireSummariesRss"

VALID_CATEGORIES = ['Emergency Warning', 'Watch and Act', 'Advice',
                    'No Alert Level']


class TfsIncidentsFeed(GeoRssFeed):
    """Tasmania Fire Service Incidents feed."""

    def __init__(self, home_coordinates, filter_radius=None,
                 filter_categories=None):
        """Initialise this service."""
        super().__init__(home_coordinates, URL, filter_radius=filter_radius)
        self._filter_categories = filter_categories

    def _new_entry(self, home_coordinates, rss_entry, global_data):
        """Generate a new entry."""
        return TfsIncidentsFeedEntry(home_coordinates, rss_entry)

    def _filter_entries(self, entries):
        """Filter the provided entries."""
        entries = super()._filter_entries(entries)
        if self._filter_categories:
            return list(filter(lambda entry:
                               entry.category in self._filter_categories,
                               entries))
        return entries


class TfsIncidentsFeedEntry(FeedEntry):
    """Tasmania Fire Service Incidents feed entry."""

    def __init__(self, home_coordinates, rss_entry):
        """Initialise this service."""
        super().__init__(home_coordinates, rss_entry)

    @property
    def attribution(self) -> str:
        """Return the attribution of this entry."""
        return ATTRIBUTION

    @property
    def location(self) -> str:
        """Return the location of this entry."""
        return self._search_in_description(REGEXP_ATTR_LOCATION)

    @property
    def region(self) -> str:
        """Return the region of this entry."""
        return self._search_in_description(REGEXP_ATTR_REGION)

    @property
    def responsible_agency(self) -> str:
        """Return the responsible agency of this entry."""
        return self._search_in_description(REGEXP_ATTR_RESPONSIBLE_AGENCY)

    @property
    def status(self) -> str:
        """Return the status of this entry."""
        return self._search_in_description(REGEXP_ATTR_STATUS)

    @property
    def size(self) -> str:
        """Return the size of this entry."""
        return self._search_in_description(REGEXP_ATTR_SIZE)

    @property
    def type(self) -> str:
        """Return the type of this entry."""
        return self._search_in_description(REGEXP_ATTR_TYPE)

    @property
    def published(self) -> Optional[datetime]:
        """Return the published date of this entry."""
        if self._rss_entry:
            return self._rss_entry.published_date
        return None



"""
RSS XML Parser with GeoRSS support.
"""
import dateparser as dateparser
import datetime
import logging
import xmltodict
from typing import Optional

from georss_client.consts import XML_TAG_GEORSS_POLYGON, XML_TAG_GEO_LONG, \
    XML_TAG_GEO_LAT, XML_TAG_GEO_POINT, XML_TAG_GML_POS, XML_TAG_GML_POINT, \
    XML_TAG_GEORSS_WHERE, XML_TAG_GEORSS_POINT, XML_ATTR_TERM, \
    XML_TAG_CATEGORY, XML_TAG_SOURCE, XML_ATTR_TEXT, XML_TAG_ID, \
    XML_TAG_GUID, XML_TAG_ENTRY, XML_TAG_ITEM, XML_TAG_NAME, XML_TAG_AUTHOR, \
    XML_TAG_LAST_BUILD_DATE, XML_TAG_TTL, XML_TAG_LANGUAGE, \
    XML_TAG_GENERATOR, XML_TAG_COPYRIGHT, XML_TAG_DC_DATE, XML_TAG_PUB_DATE, \
    XML_TAG_PUBLISHED, XML_TAG_UPDATED, XML_TAG_LINK, XML_TAG_CONTENT, \
    XML_TAG_SUMMARY, XML_TAG_DESCRIPTION, XML_TAG_TITLE, XML_TAG_FEED, \
    XML_TAG_CHANNEL, XML_TAG_RSS

_LOGGER = logging.getLogger(__name__)

DEFAULT_NAMESPACES = {
    'http://www.w3.org/2005/Atom': None,
    'http://purl.org/dc/elements/1.1/': 'dc',
    'http://www.georss.org/georss': 'georss',
    'http://www.w3.org/2003/01/geo/wgs84_pos#': 'geo',
    'http://www.opengis.net/gml': 'gml',
    'http://www.gdacs.org/': 'gdacs',
}

KEYS_DATE = [XML_TAG_DC_DATE, XML_TAG_LAST_BUILD_DATE, XML_TAG_PUB_DATE,
             XML_TAG_PUBLISHED, XML_TAG_UPDATED]
KEYS_INT = [XML_TAG_TTL]


class XmlParser:
    """Built-in XML parser."""

    def __init__(self, additional_namespaces=None):
        """Initialise the XML parser."""
        self._namespaces = DEFAULT_NAMESPACES
        if additional_namespaces:
            self._namespaces.update(additional_namespaces)

    def parse(self, xml):
        """Parse the provided xml."""
        if xml:

            def postprocessor(path, key, value):
                """Conduct type conversion for selected keys."""
                try:
                    if key in KEYS_DATE and value:
                        return key, dateparser.parse(value)
                    if key in KEYS_INT and value:
                        return key, int(value)
                except (ValueError, TypeError) as error:
                    _LOGGER.warning("Unable to process (%s/%s): %s",
                                    key, value, error)
                return key, value

            parsed_dict = xmltodict.parse(
                xml, process_namespaces=True, namespaces=self._namespaces,
                postprocessor=postprocessor)

            if XML_TAG_RSS in parsed_dict:
                rss = parsed_dict.get(XML_TAG_RSS)
                if XML_TAG_CHANNEL in rss:
                    channel = rss.get(XML_TAG_CHANNEL)
                    return Feed(channel)
            if XML_TAG_FEED in parsed_dict:
                feed = parsed_dict.get(XML_TAG_FEED)
                return Feed(feed)

        return None


class Geometry:
    """Represents a geometry."""


class Point(Geometry):
    """Represents a point."""

    def __init__(self, latitude, longitude):
        """Initialise point."""
        self._latitude = latitude
        self._longitude = longitude

    def __repr__(self):
        """Return string representation of this point."""
        return '<{}(latitude={}, longitude={})>'.format(
            self.__class__.__name__, self.latitude, self.longitude)

    @property
    def latitude(self) -> Optional[float]:
        """Return the latitude of this point."""
        return self._latitude

    @property
    def longitude(self) -> Optional[float]:
        """Return the longitude of this point."""
        return self._longitude


class Polygon(Geometry):
    """Represents a polygon."""

    def __init__(self, points):
        """Initialise polygon."""
        self._points = points

    def __repr__(self):
        """Return string representation of this polygon."""
        return '<{}(centroid={})>'.format(
            self.__class__.__name__, self.centroid)

    @property
    def points(self) -> Optional[list]:
        """Return the points of this polygon."""
        return self._points

    @property
    def centroid(self) -> Point:
        """Find the polygon's centroid as a best approximation."""
        longitudes_list = [point.longitude for point in self.points]
        latitudes_list = [point.latitude for point in self.points]
        number_of_points = len(self.points)
        longitude = sum(longitudes_list) / number_of_points
        latitude = sum(latitudes_list) / number_of_points
        return Point(latitude, longitude)


class FeedDictSource:
    """Represents a subset of a feed based on a dict."""

    def __init__(self, source):
        """Initialise feed."""
        self._source = source

    def __repr__(self):
        """Return string representation of this feed item."""
        return '<{}({})>'.format(
            self.__class__.__name__, self.link)

    def _attribute(self, names):
        """Get an attribute from this feed or feed item."""
        if self._source and names:
            # Try each name, and return the first value that is not None.
            for name in names:
                value = self._source.get(name, None)
                if value:
                    return value
        return None

    @property
    def title(self) -> Optional[str]:
        """Return the title of this feed or feed item."""
        return self._attribute([XML_TAG_TITLE])

    @property
    def description(self) -> Optional[str]:
        """Return the description of this feed or feed item."""
        return self._attribute([XML_TAG_DESCRIPTION,
                                XML_TAG_SUMMARY,
                                XML_TAG_CONTENT])

    @property
    def link(self) -> Optional[str]:
        """Return the link of this feed or feed item."""
        return self._attribute([XML_TAG_LINK])

    @property
    def published_date(self) -> Optional[datetime.datetime]:
        """Return the published date of this feed or feed item."""
        return self._attribute([XML_TAG_PUB_DATE,
                                XML_TAG_PUBLISHED,
                                XML_TAG_DC_DATE])

    @property
    def updated_date(self) -> Optional[datetime.datetime]:
        """Return the updated date of this feed or feed item."""
        return self._attribute([XML_TAG_UPDATED])

    def get_additional_attribute(self, name):
        """Get an additional attribute not provided as property."""
        return self._attribute([name])


class Feed(FeedDictSource):
    """Represents a feed."""

    @property
    def author(self) -> Optional[str]:
        """Return the author of this feed."""
        # <author>
        #   <name>Istituto Nazionale di Geofisica e Vulcanologia</name>
        #   <uri>http://www.ingv.it</uri>
        # </author>
        author = self._attribute([XML_TAG_AUTHOR])
        if author:
            name = author.get(XML_TAG_NAME, None)
            return name
        return None

    @property
    def copyright(self) -> Optional[str]:
        """Return the copyright of this feed."""
        return self._attribute([XML_TAG_COPYRIGHT])

    @property
    def generator(self) -> Optional[str]:
        """Return the generator of this feed."""
        return self._attribute([XML_TAG_GENERATOR])

    @property
    def language(self) -> Optional[str]:
        """Return the language of this feed."""
        return self._attribute([XML_TAG_LANGUAGE])

    @property
    def last_build_date(self) -> Optional[datetime.datetime]:
        """Return the last build date of this feed."""
        return self._attribute([XML_TAG_LAST_BUILD_DATE])

    @property
    def ttl(self) -> Optional[int]:
        """Return the ttl of this feed."""
        return self._attribute([XML_TAG_TTL])

    @property
    def entries(self):
        """Return the entries of this feed."""
        items = self._attribute([XML_TAG_ITEM, XML_TAG_ENTRY])
        entries = []
        if items and isinstance(items, list):
            for item in items:
                entries.append(FeedItem(item))
        else:
            # A single item in the feed is not represented as an array.
            entries.append(FeedItem(items))
        return entries


class FeedItem(FeedDictSource):
    """Represents a feed item."""

    def __repr__(self):
        """Return string representation of this feed item."""
        return '<{}({})>'.format(
            self.__class__.__name__, self.guid)

    @property
    def guid(self) -> Optional[str]:
        """Return the guid of this feed item."""
        guid = self._attribute([XML_TAG_GUID, XML_TAG_ID])
        if guid and isinstance(guid, dict) and XML_ATTR_TEXT in guid:
            # <guid isPermaLink="false">
            #   1234
            # </guid>
            guid = guid.get(XML_ATTR_TEXT)
        return guid

    @property
    def source(self) -> Optional[str]:
        """Return the source of this feed item."""
        return self._attribute([XML_TAG_SOURCE])

    @property
    def category(self) -> Optional[str]:
        """Return the category of this feed item."""
        category = self._attribute([XML_TAG_CATEGORY])
        if category and XML_ATTR_TERM in category:
            # <category term="Category 1"/>
            category = category.get(XML_ATTR_TERM)
        return category

    @property
    def geometry(self) -> Optional[Geometry]:
        """Return the geometry of this feed item."""
        # <georss:point>-0.5 119.8</georss:point>
        point = self._attribute([XML_TAG_GEORSS_POINT])
        if point:
            latitude = float(point.split(' ')[0])
            longitude = float(point.split(' ')[1])
            return Point(latitude, longitude)
        # <georss:where>
        #   <gml:Point>
        #     <gml:pos>44.11 -66.23</gml:pos>
        #   </gml:Point>
        # </georss:where>
        where = self._attribute([XML_TAG_GEORSS_WHERE])
        if where:
            point = where.get(XML_TAG_GML_POINT)
            if point:
                pos = point.get(XML_TAG_GML_POS)
                if pos:
                    latitude = float(pos.split(' ')[0])
                    longitude = float(pos.split(' ')[1])
                    return Point(latitude, longitude)
        # <geo:Point xmlns:geo="http://www.w3.org/2003/01/geo/wgs84_pos#">
        #   <geo:lat>38.3728</geo:lat>
        #   <geo:long>15.7213</geo:long>
        # </geo:Point>
        point = self._attribute([XML_TAG_GEO_POINT])
        if point:
            latitude = float(point.get(XML_TAG_GEO_LAT))
            longitude = float(point.get(XML_TAG_GEO_LONG))
            return Point(latitude, longitude)
        # <geo:long>119.948006</geo:long>
        # <geo:lat>-23.126413</geo:lat>
        lat = self._attribute([XML_TAG_GEO_LAT])
        long = self._attribute([XML_TAG_GEO_LONG])
        if long and lat:
            longitude = float(long)
            latitude = float(lat)
            return Point(latitude, longitude)
        # <georss:polygon>
        #   -34.937663524 148.597260613
        #   -34.9377026399999 148.597169138
        #   -34.9377002169999 148.59708737
        #   -34.9376945989999 148.59705595
        #   -34.9376863529999 148.596955098
        #   -34.937663524 148.597260613
        # </georss:polygon>
        polygon = self._attribute([XML_TAG_GEORSS_POLYGON])
        if polygon:
            # For now, only supporting the first polygon.
            if isinstance(polygon, list):
                polygon = polygon[0]
            # Extract coordinate pairs.
            coordinate_values = polygon.split()
            points = []
            for i in range(0, len(coordinate_values), 2):
                latitude = float(coordinate_values[i])
                longitude = float(coordinate_values[i+1])
                points.append(Point(latitude, longitude))
            return Polygon(points)
        # None of the above
        return None



"""
QFES Bushfire Alert Feed.

Fetches GeoRSS feed from QFES Bushfire Alert Feed.
"""
from datetime import datetime

from typing import Optional

from georss_client import GeoRssFeed, FeedEntry, ATTR_ATTRIBUTION
from georss_client.consts import CUSTOM_ATTRIBUTE

REGEXP_ATTR_STATUS = 'Current Status: (?P<{}>[^<]+)[\n\r]'\
    .format(CUSTOM_ATTRIBUTE)

URL = "https://www.qfes.qld.gov.au/data/alerts/bushfireAlert.xml"

VALID_CATEGORIES = ['Emergency Warning', 'Watch and Act', 'Advice',
                    'Notification']


class QfesBushfireAlertFeed(GeoRssFeed):
    """QFES Bushfire Alert feed."""

    def __init__(self, home_coordinates, filter_radius=None,
                 filter_categories=None):
        """Initialise this service."""
        super().__init__(home_coordinates, URL, filter_radius=filter_radius)
        self._filter_categories = filter_categories

    def _new_entry(self, home_coordinates, rss_entry, global_data):
        """Generate a new entry."""
        attribution = None if not global_data and ATTR_ATTRIBUTION not in \
            global_data else global_data[ATTR_ATTRIBUTION]
        return QfesBushfireAlertFeedEntry(home_coordinates, rss_entry,
                                          attribution)

    def _filter_entries(self, entries):
        """Filter the provided entries."""
        entries = super()._filter_entries(entries)
        if self._filter_categories:
            return list(filter(lambda entry:
                               entry.category in self._filter_categories,
                               entries))
        return entries


class QfesBushfireAlertFeedEntry(FeedEntry):
    """QFES Bushfire Alert feed entry."""

    def __init__(self, home_coordinates, rss_entry, attribution):
        """Initialise this service."""
        super().__init__(home_coordinates, rss_entry)
        self._attribution = attribution

    @property
    def attribution(self) -> str:
        """Return the attribution of this entry."""
        return self._attribution

    @property
    def status(self) -> str:
        """Return the status of this entry."""
        return self._search_in_description(REGEXP_ATTR_STATUS)

    @property
    def published(self) -> Optional[datetime]:
        """Return the published date of this entry."""
        if self._rss_entry:
            return self._rss_entry.published_date
        return None

    @property
    def updated(self) -> Optional[datetime]:
        """Return the updated date of this entry."""
        if self._rss_entry:
            return self._rss_entry.updated_date
        return None



"""
Generic GeoRSS feed.

Support for generic GeoRSS feeds from various sources.
"""
from georss_client import GeoRssFeed, FeedEntry
from georss_client.consts import ATTR_ATTRIBUTION


class GenericFeed(GeoRssFeed):
    """Generic GeoJSON feed."""

    def __init__(self, home_coordinates, url, filter_radius=None,
                 filter_categories=None):
        """Initialise this service."""
        super().__init__(home_coordinates, url, filter_radius=filter_radius,
                         filter_categories=filter_categories)

    def _new_entry(self, home_coordinates, rss_entry, global_data):
        """Generate a new entry."""
        attribution = None if not global_data and ATTR_ATTRIBUTION not in \
            global_data else global_data[ATTR_ATTRIBUTION]
        return GenericFeedEntry(home_coordinates, rss_entry, attribution)


class GenericFeedEntry(FeedEntry):
    """Generic feed entry."""

    def __init__(self, home_coordinates, rss_entry, attribution):
        """Initialise this service."""
        super().__init__(home_coordinates, rss_entry)
        self._attribution = attribution

    @property
    def attribution(self) -> str:
        """Return the attribution of this entry."""
        return self._attribution
